
##
#   Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.
#
#   File:      sensitivity.cs
#
#   Purpose:   To demonstrate how to perform sensitivity
#   analysis from the API on a small problem:
#
#   minimize
#
#   obj: +1 x11 + 2 x12 + 5 x23 + 2 x24 + 1 x31 + 2 x33 + 1 x34
#   st
#   c1:   +  x11 +   x12                                           <= 400
#   c2:                  +   x23 +   x24                           <= 1200
#   c3:                                  +   x31 +   x33 +   x34   <= 1000
#   c4:   +  x11                         +   x31                   = 800
#   c5:          +   x12                                           = 100
#   c6:                  +   x23                 +   x33           = 500
#   c7:                          +   x24                 +   x34   = 500
#
#   The example uses basis type sensitivity analysis.
##


import sys

import pymosek as mosek

try:
    from Numeric import array, Float, zeros, ones
except ImportError:
    from mosekarr import array, Float, zeros, ones


# Since the actual value of Infinity is ignores, we define it solely
# for symbolic purposes:
inf = 0.0

# Define a stream printer to grab output from MOSEK
def streamprinter(text):
    sys.stdout.write(text)
    sys.stdout.flush()



# We might write everything directly as a script, but it looks nicer
# to create a function.
def main ():
    # Create a handle to MOSEK
    mskhandle = mosek.mosek ()
    # Create a MOSEK environment
    env = mskhandle.Env ()
    # Attach a printer to the environment
    env.set_Stream (mosek.streamtype.log, streamprinter)
    # Initialize the environment
    env.init ()

    # Create a task
    task = env.Task(0,0)
    # Attach a printer to the task
    task.set_Stream (mosek.streamtype.log, streamprinter)

    # Set up data

    bkc = array([ mosek.boundkey.up,mosek.boundkey.up,
                  mosek.boundkey.up,mosek.boundkey.fx,
                  mosek.boundkey.fx,mosek.boundkey.fx,
                  mosek.boundkey.fx ])
    blc  = array([ -inf,  -inf,  -inf,  800., 100., 500., 500. ])
    buc  = array([  400., 1200., 1000., 800., 100., 500., 500. ])

    bkx = array([ mosek.boundkey.lo,mosek.boundkey.lo,
                  mosek.boundkey.lo,mosek.boundkey.lo,
                  mosek.boundkey.lo,mosek.boundkey.lo,
                  mosek.boundkey.lo ])
    c   = array([ 1.0,2.0,5.0,2.0,1.0,2.0,1.0 ])
    blx = array([ 0.0,0.0,0.0,0.0,0.0,0.0,0.0 ])
    bux = array([ inf,inf,inf,inf,inf,inf,inf ])

    ptrb = array([ 0,2,4,6, 8,10,12 ])
    ptre = array([ 2,4,6,8,10,12,14 ])
    sub  = array([ 0,3,0,4,1,5,1,6,2,3,2,5,2,6 ])

    val  = array([ 1.0,1.0,1.0,1.0,1.0,1.0,1.0,
                   1.0,1.0,1.0,1.0,1.0,1.0,1.0 ])

    numcon = len(bkc)
    numvar = len(bkx)
    numanz = len(val)

    # Input linear data
##REF:func:task:inputdata
    task.inputdata(numcon,numvar,
                   c,0.0,
                   ptrb, ptre,  sub,  val,
                   bkc, blc, buc,
                   bkx, blx, bux)
    # Set objective sense            
    task.putobjsense(mosek.objsense.minimize)

    # Optimize
##REF:func:task:optimize
    task.optimize();
            
    # Analyze upper bound on c1 and the equality constraint on c4
    subi  = array([ 0, 3 ])
    marki = array([ mosek.mark.up, mosek.mark.up ])

    # Analyze lower bound on the variables x12 and x31
    subj  = array([ 1, 4 ])
    markj = array([ mosek.mark.lo, mosek.mark.lo ])

    leftpricei  = zeros(2,Float)
    rightpricei = zeros(2,Float)
    leftrangei  = zeros(2,Float)
    rightrangei = zeros(2,Float)
    leftpricej  = zeros(2,Float)
    rightpricej = zeros(2,Float)
    leftrangej  = zeros(2,Float)
    rightrangej = zeros(2,Float)

        
##REF:func:task:primalsensitivity
    task.primalsensitivity( subi, 
                            marki,  
                            subj,
                            markj, 
                            leftpricei,   
                            rightpricei,
                            leftrangei, 
                            rightrangei,
                            leftpricej,   
                            rightpricej,
                            leftrangej, 
                            rightrangej)

    print 'Results from sensitivity analysis on bounds:'
    print '\tleftprice  | rightprice | leftrange  | rightrange ' 
    print 'For constraints:'

    for i in range(2):
        print '\t%10f   %10f   %10f   %10f' % (leftpricei[i],
                                               rightpricei[i],
                                               leftrangei[i],
                                               rightrangei[i])
                
    print 'For variables:'
    for i in range(2):
        print '\t%10f   %10f   %10f   %10f' % (leftpricej[i],
                                               rightpricej[i],
                                               leftrangej[i],
                                               rightrangej[i])


    leftprice  = zeros(2,Float)
    rightprice = zeros(2,Float)
    leftrange  = zeros(2,Float)
    rightrange = zeros(2,Float)
    subc       = array([ 2, 5 ])
       
##REF:func:task:dualsensitivity
    task.dualsensitivity( subc,
                          leftprice,
                          rightprice,
                          leftrange,
                          rightrange)

    print 'Results from sensitivity analysis on objective coefficients:'
    
    for i in range(2):
        print '\t%10f   %10f   %10f   %10f' % (leftprice[i],
                                               rightprice[i],
                                               leftrange[i],
                                               rightrange[i])

    return None

# call the main function
try:
    main ()
except mosek.Exception, e:
    print "ERROR: %s" % str(e.errno)
    if e.msg is not None:
        print "\t%s" % e.msg
    sys.exit(1)
except:
    import traceback
    traceback.print_exc()
    sys.exit(1)
